/* Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.qpid.example.jmsexample.pubsub;

import java.util.Properties;

import javax.jms.BytesMessage;
import javax.jms.ConnectionFactory;
import javax.jms.ExceptionListener;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.jms.Topic;
import javax.jms.TopicConnection;
import javax.jms.TopicSession;
import javax.naming.Context;
import javax.naming.InitialContext;

/**
 * The example creates a TopicSubscriber on the specified
 * Topic and uses a MessageListener with this TopicSubscriber
 * in order to enable asynchronous delivery.
 */
public class Subscriber
{
    /* Used in log output. */
    private static final String CLASS="Subscriber";

    /**
     * Run the message consumer example.
     *
     * @param args Command line arguments.
     */
    public static void main(String[] args)
    {
        Subscriber listener=new Subscriber();
        listener.runTest();
    }


    /**
     * Start the example.
     */
    private void runTest()
    {
        try
        {
            Properties properties=new Properties();
            properties.load(this.getClass().getResourceAsStream("pubsub.properties"));

            //Create the initial context
            Context ctx=new InitialContext(properties);

            // Declare the connection
            ConnectionFactory conFac=(ConnectionFactory) ctx.lookup("qpidConnectionfactory");
            TopicConnection connection=(TopicConnection) conFac.createConnection();

            // As this application is using a TopicSubscriber we need to set an ExceptionListener on the connection
            // so that errors raised within the JMS client library can be reported to the application
            System.out.println(
                    CLASS + ": Setting an ExceptionListener on the connection as sample uses a TopicSubscriber");

            connection.setExceptionListener(new ExceptionListener()
            {
                public void onException(JMSException jmse)
                {
                    // The connection may have broken invoke reconnect code if available.
                    System.err.println(CLASS + ": The sample received an exception through the ExceptionListener");
                    System.exit(0);
                }
            });

            // Create a session on the connection
            // This session is a default choice of non-transacted and uses
            // the auto acknowledge feature of a session.
            //TopicSession session=connection.createTopicSession(false, Session.AUTO_ACKNOWLEDGE);
            //TopicSession session=connection.createTopicSession(false, Session.CLIENT_ACKNOWLEDGE);
            TopicSession session=connection.createTopicSession(true, Session.AUTO_ACKNOWLEDGE);

            Topic topic=(Topic) ctx.lookup("usa.weather");
            System.out.println(CLASS + ": Creating a Message Subscriber for topic usa.weather");
            //javax.jms.TopicSubscriber messageSubscriber=session.createSubscriber(topic);
            javax.jms.TopicSubscriber messageSubscriber=session.createDurableSubscriber(topic, "durable");

            // Now the messageConsumer is set up we can start the connection
            System.out.println(CLASS + ": Starting connection so TopicSubscriber can receive messages");
            connection.start();

	    int i = 0;
            while(i < 10)
            {
                Message message = messageSubscriber.receive();
                if (message != null)
                {
                System.out.println(CLASS + ": Received message: "+message);
                i++;

                if (message instanceof TextMessage)
                {
                    String text=((TextMessage) message).getText();
                }
                else
                {
                    byte[] body=new byte[(int) ((BytesMessage) message).getBodyLength()];
                    ((BytesMessage) message).readBytes(body);
                    String text=new String(body);
                }
                //session.commit();
                session.rollback();
                }
            }

            System.out.println(CLASS + ": Closing connection");
            connection.close();

            // Close the JNDI reference
            System.out.println(CLASS + ": Closing JNDI context");
            ctx.close();
        }
        catch (Exception exp)
        {
            exp.printStackTrace();
            System.err.println(CLASS + ": Caught an Exception: " + exp);
        }
    }
}
